@@ -1,4 +1,4 @@
-;;; haskell-indent.el --- "semi-intelligent" indentation module for Haskell Mode
+;;; haskell-indent.el --- "semi-intelligent" indentation module for Haskell Mode -*- lexical-binding: t -*-
 
 ;; Copyright 2004, 2005, 2007, 2008, 2009  Free Software Foundation, Inc.
 ;; Copyright 1997-1998  Guy Lapalme
@@ -92,24 +92,28 @@
 
 (defvar haskell-literate)
 
+;;;###autoload
 (defgroup haskell-indent nil
   "Haskell indentation."
   :group 'haskell
   :link '(custom-manual "(haskell-mode)Indentation")
   :prefix "haskell-indent-")
 
+;;;###autoload
 (defcustom haskell-indent-offset 4
   "Indentation of Haskell statements with respect to containing block."
   :type 'integer
   :safe #'natnump
   :group 'haskell-indent)
 
+;;;###autoload
 (defcustom haskell-indent-literate-Bird-default-offset 1
   "Default number of blanks after > in a Bird style literate script."
   :type 'integer
   :safe #'natnump
   :group 'haskell-indent)
 
+;;;###autoload
 (defcustom haskell-indent-rhs-align-column 0
   "Column on which to align right-hand sides (use 0 for ad-hoc alignment)."
   :type 'integer
@@ -322,7 +326,7 @@ It deals with both Bird style and non Bird-style scripts."
         (insert "\\begin{code}\n")))))
 
 ;;; Start of indentation code
-
+;;;###autoload
 (defcustom haskell-indent-look-past-empty-line t
   "If nil, indentation engine will not look past an empty line for layout points."
   :group 'haskell-indent
@@ -436,7 +440,7 @@ Returns the location of the start of the comment, nil otherwise."
 
 (defun haskell-indent-next-symbol-safe (end)
   "Puts point to the next following symbol, or to end if there are no more symbols in the sexp."
-  (condition-case errlist (haskell-indent-next-symbol end)
+  (condition-case _errlist (haskell-indent-next-symbol end)
     (error (goto-char end))))
 
 (defun haskell-indent-separate-valdef (start end)
@@ -1018,6 +1022,7 @@ OPEN is the start position of the comment in which point is."
                             (haskell-indent-point-to-col (match-end 0)))
                         (haskell-indent-point-to-col (point))))))))))
 
+;;;###autoload
 (defcustom haskell-indent-thenelse 0
   "If non-nil, \"then\" and \"else\" are indented.
 This is necessary in the \"do\" layout under Haskell-98.
@@ -1049,6 +1054,7 @@ See http://hackage.haskell.org/trac/haskell-prime/wiki/DoAndIfThenElse"
     (list (list (+ (if (memq (char-after) '(?t ?e)) haskell-indent-thenelse 0)
                    (haskell-indent-point-to-col open))))))
 
+;;;###autoload
 (defcustom haskell-indent-after-keywords
   '(("where" 2 0)
     ("of" 2)
@@ -1097,6 +1103,7 @@ is at the end of an otherwise-non-empty line."
     (or (assoc id haskell-indent-after-keywords)
         (car (member id haskell-indent-after-keywords)))))
 
+;;;###autoload
 (defcustom haskell-indent-dont-hang '("(")
   "Lexemes that should never be considered as hanging."
   :group 'haskell-indent
@@ -1295,7 +1302,7 @@ We stay in the cycle as long as the TAB key is pressed."
         (if marker
             (goto-char (marker-position marker)))))))
 
-(defun haskell-indent-region (start end)
+(defun haskell-indent-region (_start _end)
   (error "Auto-reindentation of a region is not supported"))
 
 ;;; alignment functions
@@ -1433,7 +1440,7 @@ TYPE is either 'guard or 'rhs."
             (if regstack
                 (haskell-indent-shift-columns maxcol regstack)))))))
 
-(defun haskell-indent-align-guards-and-rhs (start end)
+(defun haskell-indent-align-guards-and-rhs (_start _end)
   "Align the guards and rhs of functions in the region, which must be active."
   ;; The `start' and `end' args are dummys right now: they're just there so
   ;; we can use the "r" interactive spec which properly signals an error.
@@ -1513,6 +1520,10 @@ One indentation cycle is used."
 ;;;###autoload
 (defun turn-on-haskell-indent ()
   "Turn on ``intelligent'' Haskell indentation mode."
+  (when (and (bound-and-true-p haskell-indentation-mode)
+             (fboundp 'haskell-indentation-mode))
+    (haskell-indentation-mode 0))
+
   (set (make-local-variable 'indent-line-function) 'haskell-indent-cycle)
   (set (make-local-variable 'indent-region-function) 'haskell-indent-region)
   (setq haskell-indent-mode t)
